package METRICS;

# ------------------------------------------------------------------------

#Copyright (C) 2005 Enrique Amig and Jess Gimnez

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License as published by the Free Software Foundation; either
#version 2.1 of the License, or (at your option) any later version.

#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Lesser General Public License for more details.

#You should have received a copy of the GNU Lesser General Public
#License along with this library; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# ------------------------------------------------------------------------

use strict;
use Data::Dumper;
use IQ::IQXML;
use IQ::QARLA;

use vars qw($SGMLEXT $GTMEXT $NISTEXT $METEXT $BLEUEXT $MBLEUEXT $WEREXT $PEREXT $ROUGEXT $REPORTEXT $SRCEXT $REFEXT $CFGEXT $XMLEXT $GZEXT $version);

#FILE EXTENSIONS
$METRICS::SGMLEXT = "sgml";
$METRICS::GTMEXT = "GTM";
$METRICS::NISTEXT = "NIST";
$METRICS::BLEUEXT = "BLEU";
$METRICS::MBLEUEXT = "MBLEU";
$METRICS::METEXT = "MTR";
$METRICS::WEREXT = "WER";
$METRICS::PEREXT = "PER";
$METRICS::ROUGEXT = "RG";
$METRICS::REPORTEXT = "report";
$METRICS::SRCEXT = "src";
$METRICS::REFEXT = "ref";
$METRICS::CFGEXT = "config";
$METRICS::XMLEXT = "xml";
$METRICS::GZEXT = "gz";

#available metrics
$METRICS::TBLEU = "mteval-kit/mteval-v11b.pl -b -d 2"; # version v10
$METRICS::TGTM = "GTM/gtm-1.2-binary/gtm.jar +s +d";   # version 1.2
$METRICS::TMETEOR = "METEOR/meteor.pl";                # version 0.4.3
$METRICS::TNIST = "mteval-kit/mteval-v11b.pl -n -d 2"; # version v10
$METRICS::TROUGE = "ROUGE/ROUGE-1.5.5.pl";             # version 1.5.5
$METRICS::TROUGEdata = "ROUGE/data";                   # version 1.5.5
$METRICS::TWER = "WER/WER.pl";
$METRICS::TPER = "PER/PER.pl";

#application details
$METRICS::VERSION = "1.0";
$METRICS::appversion = "1.0";
$METRICS::appname = "IQMT";
$METRICS::appyear = "2005";

my $EMPTY_ITEM = "***EMPTY***";

sub criterion {
  my $A = substr($a, 0, 1); 
  my $B = substr($b, 0, 1);
  if ($A eq $B) { my @lA = split(/[RDS]/, $a); my @lB = split(/[RDS]/, $b); return ($lA[1] <=> $lB[1]); }
  else { return ($A cmp $B); } 
}

sub read_mbleu {
   #description _ read BLEU value from report file
   #param1  _ report filename
   #@return _ BLEU score list (n = 1..9)

   my $report = shift;

   open(AUX, "<$report") or die "couldn't open file: $report\n";
   my $aux = <AUX>; close(AUX); chomp($aux);
   my @lbleu = split(" ", $aux);

   return $lbleu[2];
}

sub read_bleu {
   #description _ read BLEU value from report file
   #param1  _ report filename
   #@return _ BLEU score list (n = 1..9)

   my $report = shift;

   my @lbleu;
   open(AUX, "tail -n 3 $report | grep 'BLEU' |") or die "couldn't open file: $report\n";
   my $aux = <AUX>; close(AUX); chomp($aux);
   $aux =~ s/^ +BLEU: +//; $aux =~ s/ +/ /g;
   my @lbleu = split(" ", $aux);
   pop(@lbleu);

   return @lbleu;
}

sub read_bleu_segments {
   #description _ read BLEU-4 value from report file (for all segments)
   #param1  _ report filename
   #@return _ bleu score list

   my $report = shift;

   my @lbleu1;
   my @lbleu2;
   my @lbleu3;
   my @lbleu4;

   open(AUX, "grep ' BLEU score using' $report |") or die "couldn't open file: $report\n";
   while (my $aux = <AUX>) {
      chomp($aux);
      my @laux = split(/ +/, $aux);
      my @lN;
      push(@lbleu1, $laux[6]);
      push(@lbleu2, $laux[7]);
      push(@lbleu3, $laux[8]);
      push(@lbleu4, $laux[9]);
   }
   close(AUX);

   return (\@lbleu1, \@lbleu2, \@lbleu3, \@lbleu4);
}

sub read_gtm {
   #description _ read GTM value from report file
   #param1  _ report filename
   #@return _ GTM score

   my $report = shift;

   open(AUX, "tail -n 1 $report |") or die "couldn't open file: $report\n";
   my $gtm = <AUX>;
   close(AUX);
   chomp($gtm);
   $gtm =~ s/^ +//;

   return $gtm;
}

sub read_gtm_segments {
   #description _ read GTM value from report file (for all segments)
   #param1  _ report filename
   #@return _ gtm F1 score list

   my $report = shift;

   my @lgtm;

   open(AUX, "grep '\"' $report |") or die "couldn't open file: $report\n";
   while (my $aux = <AUX>) {
      chomp($aux);
      my @laux = split(/ +/, $aux);
      push(@lgtm, $laux[2]);
   }
   close(AUX);

   #print Dumper(@lgtm);

   return \@lgtm;
}

sub read_nist {
   #description _ read NIST value from report file
   #param1  _ report filename
   #@return _ NIST score list (n = 1..9)

   my $report = shift;

   my @lnist;
   open(AUX, "tail -n 3 $report | grep 'NIST' |") or die "couldn't open file: $report\n";
   my $aux = <AUX>; close(AUX); chomp($aux);
   $aux =~ s/^ +NIST: +//; $aux =~ s/ +/ /g;
   my @lnist = split(" ", $aux);
   pop(@lnist);

   return @lnist;
}

sub read_nist_segments {
   #description _ read NIST-5 value from report file (for all segments)
   #param1  _ report filename
   #@return _ NIST score list

   my $report = shift;

   my @lnist1;
   my @lnist2;
   my @lnist3;
   my @lnist4;
   my @lnist5;

   open(AUX, "grep ' NIST score using' $report |") or die "couldn't open file: $report\n";
   while (my $aux = <AUX>) {
      chomp($aux);
      my @laux = split(/ +/, $aux);
      push(@lnist1, $laux[6]);
      push(@lnist2, $laux[7]);
      push(@lnist3, $laux[8]);
      push(@lnist4, $laux[9]);
      push(@lnist5, $laux[10]);
   }
   close(AUX);

   return (\@lnist1, \@lnist2, \@lnist3, \@lnist4, \@lnist5);
}

sub read_meteor {
   #description _ read METEOR value from report file
   #param1  _ report filename
   #@return _ METEOR score

   my $report = shift;

   open(AUX, "tail -n 15 $report | grep '^Score:' |") or die "couldn't open file: $report\n";
   my $aux = <AUX>; close(AUX); chomp($aux);
   my @lmeteor = split(" ", $aux);

   return $lmeteor[1];
}

sub read_meteor_segments {
   #description _ read METEOR value from report file (for all segments)
   #param1  _ report filename
   #@return _ METEOR score list

   my $report = shift;

   my @lMET;

   open(AUX, "grep '^sgmScore:' $report |") or die "couldn't open file: $report\n";
   open(AUX2, "grep '^Document:' $report |") or die "couldn't open file: $report\n";

   while (my $aux = <AUX>) {
      chomp($aux);
      my @laux = split(/ +/, $aux);
      my $aux2 = <AUX2>;
      chomp($aux2);
      my @laux2 = split(/ +/, $aux2);
      my $index = $laux2[3];
      $lMET[$index-1] = $laux[1];
      #push(@lMET, $laux[1]);
   }
   close(AUX);
   close(AUX2);

   #print Dumper(@lMET);

   return \@lMET;
}

sub read_WER {
   #description _ read WER value from report file
   #param1  _ report filename
   #@return _ WER

   my $report = shift;

   open(AUX, "grep 'WER score' $report |") or die "couldn't open file: $report\n";
   my $aux = <AUX>; close(AUX);
   my @lWER = split(" ", $aux);

   return $lWER[3];
}

sub read_WER_segments {
   #description _ read WER value from report file (for all segments)
   #param1  _ report filename
   #@return _ WER score list

   my $report = shift;

   my @lWER;

   open(AUX, "grep 'WERsgm score' $report |") or die "couldn't open file: $report\n";
   while (my $aux = <AUX>) {
      chomp($aux);
      my @laux = split(/ +/, $aux);
      push(@lWER, 1 - $laux[3]);
   }
   close(AUX);

   #print Dumper(@lWER);

   return \@lWER;
}

sub read_PER {
   #description _ read PER value from report file
   #param1  _ report filename
   #@return _ PER

   my $report = shift;

   open(AUX, "grep 'PER score' $report |") or die "couldn't open file: $report\n";
   my $aux = <AUX>; close(AUX);
   my @lPER = split(" ", $aux);

   return $lPER[3];
}

sub read_PER_segments {
   #description _ read PER value from report file (for all segments)
   #param1  _ report filename
   #@return _ PER score list

   my $report = shift;

   my @lPER;

   open(AUX, "grep 'PERsgm score' $report |") or die "couldn't open file: $report\n";
   while (my $aux = <AUX>) {
      chomp($aux);
      my @laux = split(/ +/, $aux);
      push(@lPER, 1 - $laux[3]);
   }
   close(AUX);

   #print Dumper(@lPER);

   return \@lPER;
}

sub read_rouge {
   #description _ read ROUGE value from report file (for all segments)
   #param1  _ report filename
   #@return _ ROUGE score list

   my $report = shift;

   my %Hrouge;
   my @lrouge;

   open(AUX, "grep 'X ROUGE-[SLW1-4][^ ]* Average_F' $report |") or die "couldn't open file: $report\n";
   while (my $aux = <AUX>) {
      chomp($aux);
      my @laux = split(/ +/, $aux);
      $Hrouge{$laux[1]} = $laux[3];
   }
   close(AUX);

   foreach my $s (sort keys %Hrouge) { push(@lrouge, $Hrouge{$s}); }

   return @lrouge;
}

sub read_rouge_segments {
   #description _ read ROUGE value from report file (for all segments)
   #param1  _ report filename
   #@return _ ROUGE score list

   my $report = shift;

   my @lrouge1;
   my @lrouge2;
   my @lrouge3;
   my @lrouge4;
   my @lrougeL;
   my @lrougeS;
   my @lrougeSU;
   my @lrougeW;

   open(AUX, "grep 'X ROUGE-[SLW1-4][^ ]* Eval' $report |") or die "couldn't open file: $report\n";
   while (my $aux = <AUX>) {
      chomp($aux);
      my @laux = split(/ +/, $aux);
      my @ll = split(/\./, $laux[3]);
      my @llF = split(":", $laux[6]);
      if ($laux[1] eq "ROUGE-1") { push(@lrouge1, $llF[1]); }
      if ($laux[1] eq "ROUGE-2") { push(@lrouge2, $llF[1]); }
      if ($laux[1] eq "ROUGE-3") { push(@lrouge3, $llF[1]); }
      if ($laux[1] eq "ROUGE-4") { push(@lrouge4, $llF[1]); }
      if ($laux[1] eq "ROUGE-L") { push(@lrougeL, $llF[1]); }
      if ($laux[1] eq "ROUGE-S\*") { push(@lrougeS, $llF[1]); }
      if ($laux[1] eq "ROUGE-SU\*") { push(@lrougeSU, $llF[1]); }
      if ($laux[1] eq "ROUGE-W-1.2") { push(@lrougeW, $llF[1]); }
   }
   close(AUX);

   return (\@lrouge1, \@lrouge2, \@lrouge3, \@lrouge4, \@lrougeL, \@lrougeS, \@lrougeSU, \@lrougeW);
}

# *************************************************************************************
# ************************* INDIVIDUAL METRICS FOR QARLA (multiple reference) *********
# *************************************************************************************

sub doMultiBLEU {
   #description _ computes BLEU score (by calling NIST mteval script) -> n = 1..4 (multiple reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [list reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"$BLEUEXT-1"} or $m->{"$BLEUEXT-2"} or $m->{"$BLEUEXT-3"} or $m->{"$BLEUEXT-4"}) {

      if ($verbose == 1) { print STDERR "BLEU.."; }

      my $reportBLEU1xml = "$fname.$BLEUEXT-1.$XMLEXT";
      my $reportBLEU2xml = "$fname.$BLEUEXT-2.$XMLEXT";
      my $reportBLEU3xml = "$fname.$BLEUEXT-3.$XMLEXT";
      my $reportBLEU4xml = "$fname.$BLEUEXT-4.$XMLEXT";
      my $reportBLEU1xmlgz = "$fname.$BLEUEXT-1.$XMLEXT.$GZEXT";
      my $reportBLEU2xmlgz = "$fname.$BLEUEXT-2.$XMLEXT.$GZEXT";
      my $reportBLEU3xmlgz = "$fname.$BLEUEXT-3.$XMLEXT.$GZEXT";
      my $reportBLEU4xmlgz = "$fname.$BLEUEXT-4.$XMLEXT.$GZEXT";

      if ((!(-e $reportBLEU1xml) and !(-e $reportBLEU1xmlgz)) or (!(-e $reportBLEU2xml) and !(-e $reportBLEU2xmlgz)) or (!(-e $reportBLEU3xml) and !(-e $reportBLEU3xmlgz)) or (!(-e $reportBLEU4xml) and !(-e $reportBLEU4xmlgz)) or $remakeREPORTS) {

         my $toolBLEU = "perl $tools/$METRICS::TBLEU";

         my $refNISTsgml = $REFname.".$REFEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_multidoc($Href, $refNISTsgml, 2, $verbose); }
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }
         my $reportBLEU = "$fname.$BLEUEXT.$REPORTEXT";
         if ($verbose > 1) { print STDERR "building $reportBLEU...\n"; }
         system("$toolBLEU -s $srcNISTsgml -t $outNISTsgml -r $refNISTsgml > $reportBLEU");
         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         my ($BLEU1, $BLEU2, $BLEU3, $BLEU4) = read_bleu($reportBLEU);
         my ($bleu1, $bleu2, $bleu3, $bleu4) = read_bleu_segments($reportBLEU);
         system("rm $reportBLEU");
         IQXML::write_report($TGTname, $REFname, "$BLEUEXT-1", $BLEU1, $bleu1, $reportBLEU1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$BLEUEXT-2", $BLEU2, $bleu2, $reportBLEU2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$BLEUEXT-3", $BLEU3, $bleu3, $reportBLEU3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$BLEUEXT-4", $BLEU4, $bleu4, $reportBLEU4xml, $verbose);
      }
   }
}

sub doMultiGTM {
   #description _ computes GTM F1 (by calling Proteus java gtm) -> e = 1..3 (multiple reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [list reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"$GTMEXT-1"} or $m->{"$GTMEXT-2"} or $m->{"$GTMEXT-3"}) {

      if ($verbose == 1) { print STDERR "GTM.."; }

      my $reportGTM1xml = "$fname.$GTMEXT-1.$XMLEXT";
      my $reportGTM2xml = "$fname.$GTMEXT-2.$XMLEXT";
      my $reportGTM3xml = "$fname.$GTMEXT-3.$XMLEXT";
      my $reportGTM1xmlgz = "$fname.$GTMEXT-1.$XMLEXT.$GZEXT";
      my $reportGTM2xmlgz = "$fname.$GTMEXT-2.$XMLEXT.$GZEXT";
      my $reportGTM3xmlgz = "$fname.$GTMEXT-3.$XMLEXT.$GZEXT";

      if ((!(-e $reportGTM1xml) and !(-e $reportGTM1xmlgz)) or (!(-e $reportGTM2xml) and !(-e $reportGTM2xmlgz)) or (!(-e $reportGTM3xml) and !(-e $reportGTM3xmlgz)) or $remakeREPORTS) {

         my $toolGTM = "java -jar $tools/$METRICS::TGTM";

	 my @LrefGTMsgml;
         foreach my $r (sort criterion keys %{$Href}) {
            my $ref = $Href->{$r};
            my $refGTMsgml = "$ref.$GTMEXT.$SGMLEXT";
            if ((!(-e $refGTMsgml)) or $remakeREPORTS) { IQXML::GTM_f_create_mteval_doc($ref, $refGTMsgml, $verbose); }
            push(@LrefGTMsgml, $refGTMsgml);
         }
         my $outGTMsgml = "$out.$GTMEXT.$SGMLEXT";
         if ((!(-e $outGTMsgml)) or $remakeREPORTS) { IQXML::GTM_f_create_mteval_doc($out, $outGTMsgml, $verbose); }

         my $reportGTM1 = "$fname.$GTMEXT-1.$REPORTEXT";
         my $reportGTM2 = "$fname.$GTMEXT-2.$REPORTEXT";
         my $reportGTM3 = "$fname.$GTMEXT-3.$REPORTEXT";

         if (((!(-e $reportGTM1xml) and !(-e $reportGTM1xmlgz)) or $remakeREPORTS) and $m->{"$GTMEXT-1"}) {
            if ($verbose > 1) { print STDERR "building $reportGTM1...\n"; }
            system("$toolGTM -e 1 $outGTMsgml ".join(" ", @LrefGTMsgml)." > $reportGTM1");
            my $GTM1 = read_gtm($reportGTM1);
            my $gtm1 = read_gtm_segments($reportGTM1);
            IQXML::write_report($TGTname, $REFname, "$GTMEXT-1", $GTM1, $gtm1, $reportGTM1xml, $verbose);
            system("rm $reportGTM1");
         }
         if (((!(-e $reportGTM2xml) and !(-e $reportGTM2xmlgz)) or $remakeREPORTS) and $m->{"$GTMEXT-2"}) {
            if ($verbose > 1) { print STDERR "building $reportGTM2...\n"; }
            system("$toolGTM -e 2 $outGTMsgml ".join(" ", @LrefGTMsgml)." > $reportGTM2");
            my $GTM2 = read_gtm($reportGTM2);
            my $gtm2 = read_gtm_segments($reportGTM2);
            IQXML::write_report($TGTname, $REFname, "$GTMEXT-2", $GTM2, $gtm2, $reportGTM2xml, $verbose);
            system("rm $reportGTM2");
         }
         if (((!(-e $reportGTM3xml) and !(-e $reportGTM3xmlgz)) or $remakeREPORTS) and $m->{"$GTMEXT-3"}) {
            if ($verbose > 1) { print STDERR "building $reportGTM3...\n"; }
            system("$toolGTM -e 3 $outGTMsgml ".join(" ", @LrefGTMsgml)." > $reportGTM3");
            my $GTM3 = read_gtm($reportGTM3);
            my $gtm3 = read_gtm_segments($reportGTM3);
            IQXML::write_report($TGTname, $REFname, "$GTMEXT-3", $GTM3, $gtm3, $reportGTM3xml, $verbose);
            system("rm $reportGTM3");
         }

         foreach my $r (@LrefGTMsgml) { if (-e $r) { system("rm $r"); } }
         if (-e $outGTMsgml) { system "rm $outGTMsgml"; }
      }
   }
}

sub doMultiMETEOR {
   #description _ computes METEOR scores (exact + stem + wnstem + wnsyn) (multiple reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [list reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"$METEXT-exact"} or $m->{"$METEXT-stem"} or $m->{"$METEXT-wnstm"} or $m->{"$METEXT-wnsyn"}) {

      if ($verbose == 1) { print STDERR "METEOR.."; }

      my $reportMETexactXML = "$fname.$METEXT-exact.$XMLEXT";
      my $reportMETstemXML = "$fname.$METEXT-stem.$XMLEXT";
      my $reportMETwnstmXML = "$fname.$METEXT-wnstm.$XMLEXT";
      my $reportMETwnsynXML = "$fname.$METEXT-wnsyn.$XMLEXT";
      my $reportMETexactXMLgz = "$fname.$METEXT-exact.$XMLEXT.$GZEXT";
      my $reportMETstemXMLgz = "$fname.$METEXT-stem.$XMLEXT.$GZEXT";
      my $reportMETwnstmXMLgz = "$fname.$METEXT-wnstm.$XMLEXT.$GZEXT";
      my $reportMETwnsynXMLgz = "$fname.$METEXT-wnsyn.$XMLEXT.$GZEXT";

      if ((!(-e $reportMETexactXML) and !(-e $reportMETexactXMLgz)) or (!(-e $reportMETstemXML) and !(-e $reportMETstemXMLgz)) or (!(-e $reportMETwnstmXML) and !(-e $reportMETwnstmXMLgz)) or (!(-e $reportMETwnsynXML) and !(-e $reportMETwnsynXMLgz)) or $remakeREPORTS) {

         my $toolMETEOR = "perl $tools/$METRICS::TMETEOR";

         my $refNISTsgml = $REFname.".$REFEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_multidoc($Href, $refNISTsgml, 2, $verbose); }
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }
         my $refMETsgml = $REFname.".$REFEXT.$METEXT.$SGMLEXT";
         if ((!(-e $refMETsgml)) or $remakeREPORTS) {
            system "awk '{if (\$0 ~ /seg id=\"/) { gsub(/id=\"/,\"id=\"); gsub(/\">/,\">\"); } print;}' < $refNISTsgml > $refMETsgml";
         }
         my $outMETsgml = "$out.$METEXT.$SGMLEXT";
         if ((!(-e $outMETsgml)) or $remakeREPORTS) {
            system "awk '{if (\$0 ~ /seg id=\"/) { gsub(/id=\"/,\"id=\"); gsub(/\">/,\">\"); } print;}' < $outNISTsgml > $outMETsgml";
         }

         my $reportMETexact = "$fname.$METEXT-exact.$REPORTEXT";
         my $reportMETstem = "$fname.$METEXT-stem.$REPORTEXT";
         my $reportMETwnstm = "$fname.$METEXT-wnstm.$REPORTEXT";
         my $reportMETwnsyn = "$fname.$METEXT-wnsyn.$REPORTEXT";

         if (((!(-e $reportMETexactXML) and !(-e $reportMETexactXMLgz)) or $remakeREPORTS) and $m->{"$METEXT-exact"}) { #exact
            if ($verbose > 1) { print STDERR "building $reportMETexact ...\n"; }
            system("$toolMETEOR -s dummysysTST -t $outMETsgml -r $refMETsgml --modules \"exact\" > $reportMETexact");
            my $METEOREXACT = read_meteor($reportMETexact);
            my $meteorexact = read_meteor_segments($reportMETexact);
            IQXML::write_report($TGTname, $REFname, "$METEXT-exact", $METEOREXACT, $meteorexact, $reportMETexactXML, $verbose);
            system("rm $reportMETexact");
         }
         if (((!(-e $reportMETstemXML) and !(-e $reportMETstemXMLgz)) or $remakeREPORTS) and $m->{"$METEXT-stem"}) { #exact + porter
            if ($verbose > 1) { print STDERR "building $reportMETstem...\n"; }
            system("$toolMETEOR -s dummysysTST -t $outMETsgml -r $refMETsgml > $reportMETstem");
            my $METEORSTEM = read_meteor($reportMETstem);
            my $meteorstem = read_meteor_segments($reportMETstem);
            IQXML::write_report($TGTname, $REFname, "$METEXT-stem", $METEORSTEM, $meteorstem, $reportMETstemXML, $verbose);
            system("rm $reportMETstem");
         }
         if (((!(-e $reportMETwnstmXML) and !(-e $reportMETwnstmXMLgz)) or $remakeREPORTS) and $m->{"$METEXT-wnstm"}) { #exact + porter_stem + wn_stem
            if ($verbose > 1) { print STDERR "building $reportMETwnstm ...\n"; }
            system("$toolMETEOR -s dummysysTST -t $outMETsgml -r $refMETsgml --modules \"exact porter_stem wn_stem\"  > $reportMETwnstm");
            my $METEORWNSTM = read_meteor($reportMETwnstm);
            my $meteorwnstm = read_meteor_segments($reportMETwnstm);
            IQXML::write_report($TGTname, $REFname, "$METEXT-wnstm", $METEORWNSTM, $meteorwnstm, $reportMETwnstmXML, $verbose);
            system("rm $reportMETwnstm");
         }
         if (((!(-e $reportMETwnsynXML) and !(-e $reportMETwnsynXMLgz)) or $remakeREPORTS) and $m->{"$METEXT-wnsyn"}) { #exact + porter_stem + wn_stem + wn_syn
            if ($verbose > 1) { print STDERR "building $reportMETwnsyn ...\n"; }
            system("$toolMETEOR -s dummysysTST -t $outMETsgml -r $refMETsgml --modules \"exact porter_stem wn_stem wn_synonymy\" > $reportMETwnsyn");
            my $METEORWNSYN = read_meteor($reportMETwnsyn);
            my $meteorwnsyn = read_meteor_segments($reportMETwnsyn);
            IQXML::write_report($TGTname, $REFname, "$METEXT-wnsyn", $METEORWNSYN, $meteorwnsyn, $reportMETwnsynXML, $verbose);
            system("rm $reportMETwnsyn");
         }

         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         if (-e $refMETsgml) { system "rm $refMETsgml"; }
         if (-e $outMETsgml) { system "rm $outMETsgml"; }
      }
   }
}

sub doMultiNIST {
   #description _ computes NIST score (by calling NIST mteval script) -> n = 1..5 (multiple reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [list reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"$NISTEXT-1"} or $m->{"$NISTEXT-2"} or $m->{"$NISTEXT-3"} or $m->{"$NISTEXT-4"} or $m->{"$NISTEXT-5"}) {

      if ($verbose == 1) { print STDERR "NIST.."; }

      my $reportNIST1xml = "$fname.$NISTEXT-1.$XMLEXT";
      my $reportNIST2xml = "$fname.$NISTEXT-2.$XMLEXT";
      my $reportNIST3xml = "$fname.$NISTEXT-3.$XMLEXT";
      my $reportNIST4xml = "$fname.$NISTEXT-4.$XMLEXT";
      my $reportNIST5xml = "$fname.$NISTEXT-5.$XMLEXT";
      my $reportNIST1xmlgz = "$fname.$NISTEXT-1.$XMLEXT.$GZEXT";
      my $reportNIST2xmlgz = "$fname.$NISTEXT-2.$XMLEXT.$GZEXT";
      my $reportNIST3xmlgz = "$fname.$NISTEXT-3.$XMLEXT.$GZEXT";
      my $reportNIST4xmlgz = "$fname.$NISTEXT-4.$XMLEXT.$GZEXT";
      my $reportNIST5xmlgz = "$fname.$NISTEXT-5.$XMLEXT.$GZEXT";

      if ((!(-e $reportNIST1xml) and !(-e $reportNIST1xmlgz)) or (!(-e $reportNIST2xml) and !(-e $reportNIST2xmlgz)) or (!(-e $reportNIST3xml) and !(-e $reportNIST3xmlgz)) or (!(-e $reportNIST4xml) and !(-e $reportNIST4xmlgz)) or (!(-e $reportNIST5xml) and !(-e $reportNIST5xmlgz)) or $remakeREPORTS) {

         my $toolNIST = "perl $tools/$METRICS::TNIST";
         my $refNISTsgml = $REFname.".$REFEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_multidoc($Href, $refNISTsgml, 2, $verbose); }
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }

         my $reportNIST = "$fname.$NISTEXT.$REPORTEXT";
         if ($verbose > 1) { print STDERR "building $reportNIST...\n"; }
         system("$toolNIST -s $srcNISTsgml -t $outNISTsgml -r $refNISTsgml > $reportNIST");
         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         my ($NIST1, $NIST2, $NIST3, $NIST4, $NIST5) = read_nist($reportNIST);
         my ($nist1, $nist2, $nist3, $nist4, $nist5) = read_nist_segments($reportNIST);
         system("rm $reportNIST");
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-1", $NIST1, $nist1, $reportNIST1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-2", $NIST2, $nist2, $reportNIST2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-3", $NIST3, $nist3, $reportNIST3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-4", $NIST4, $nist4, $reportNIST4xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-5", $NIST5, $nist5, $reportNIST5xml, $verbose);
      }
   }
}

sub doMultiROUGE {
   #description _ computes ROUGE scores -> n = 1..4, LCS, S*, SU*, W-1.2 (multiple reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [list reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"$ROUGEXT-1"} or $m->{"$ROUGEXT-2"} or $m->{"$ROUGEXT-3"} or $m->{"$ROUGEXT-4"} or $m->{"$ROUGEXT-L"} or $m->{"$ROUGEXT-Ss"} or $m->{"$ROUGEXT-SUs"} or $m->{"$ROUGEXT-W-1.2"}) {

      if ($verbose == 1) { print STDERR "ROUGE.."; }

      my $reportROUGE1xml = "$fname.$ROUGEXT-1.$XMLEXT";
      my $reportROUGE2xml = "$fname.$ROUGEXT-2.$XMLEXT";
      my $reportROUGE3xml = "$fname.$ROUGEXT-3.$XMLEXT";
      my $reportROUGE4xml = "$fname.$ROUGEXT-4.$XMLEXT";
      my $reportROUGELxml = "$fname.$ROUGEXT-L.$XMLEXT";
      my $reportROUGESxml = "$fname.$ROUGEXT-Ss.$XMLEXT";
      my $reportROUGESUxml = "$fname.$ROUGEXT-SUs.$XMLEXT";
      my $reportROUGEWxml = "$fname.$ROUGEXT-W-1.2.$XMLEXT";
      my $reportROUGE1xmlgz = "$fname.$ROUGEXT-1.$XMLEXT.$GZEXT";
      my $reportROUGE2xmlgz = "$fname.$ROUGEXT-2.$XMLEXT.$GZEXT";
      my $reportROUGE3xmlgz = "$fname.$ROUGEXT-3.$XMLEXT.$GZEXT";
      my $reportROUGE4xmlgz = "$fname.$ROUGEXT-4.$XMLEXT.$GZEXT";
      my $reportROUGELxmlgz = "$fname.$ROUGEXT-L.$XMLEXT.$GZEXT";
      my $reportROUGESxmlgz = "$fname.$ROUGEXT-Ss.$XMLEXT.$GZEXT";
      my $reportROUGESUxmlgz = "$fname.$ROUGEXT-SUs.$XMLEXT.$GZEXT";
      my $reportROUGEWxmlgz = "$fname.$ROUGEXT-W-1.2.$XMLEXT.$GZEXT";

      if ((!(-e $reportROUGE1xml) and !(-e $reportROUGE1xmlgz)) or (!(-e $reportROUGE2xml) and !(-e $reportROUGE2xmlgz)) or (!(-e $reportROUGE3xml) and !(-e $reportROUGE3xmlgz)) or (!(-e $reportROUGE4xml) and !(-e $reportROUGE4xmlgz)) or (!(-e $reportROUGELxml) and !(-e $reportROUGELxmlgz)) or (!(-e $reportROUGESxml) and !(-e $reportROUGESxmlgz)) or (!(-e $reportROUGESUxml) and !(-e $reportROUGESUxmlgz)) or (!(-e $reportROUGEWxml) and !(-e $reportROUGEWxmlgz)) or $remakeREPORTS) {

         my $toolROUGE = "$tools/$METRICS::TROUGE -e $tools/$METRICS::TROUGEdata -z SPL -2 -1 -U -r 1000 -n 4 -w 1.2 -c 95 -d"; #always with stemming
         my $reportROUGE = "$fname.$ROUGEXT.$REPORTEXT";
         my $configROUGE = "$fname.$ROUGEXT.$CFGEXT";

         open(OUT, "< $out") or die "couldn't open file: $out\n";
         open(CFG, "> $configROUGE") or die "couldn't open file: $configROUGE\n";
         if ($verbose > 1) { print STDERR "generating files for ROUGE"; }
         foreach my $r (sort criterion keys %{$Href}) {
            my $ref = $Href->{$r};
            open(REF, "< $ref") or die "couldn't open file: $ref\n";
            my $j = 1;
            while (my $auxREF = <REF>) {
               chomp($auxREF);
               my $ref_j = "$ref.$j";
               if (!(-e $ref_j) or $remakeREPORTS) {
                  open(REFj, "> $ref_j") or die "couldn't open file: $ref_j\n";
                  $auxREF =~ s/^ +//g; $auxREF =~ s/ +$//g;
                  if (($auxREF eq "") or ($auxREF =~ /^[\?\!\.]$/)){ $auxREF = $EMPTY_ITEM; }
                  print REFj $auxREF, "\n";
                  close(REFj);
               }
               $j++;
            }
            close(REF);
         }
         my $i = 1;
         while (my $auxOUT = <OUT>) {
            chomp($auxOUT);
            my @cfgline;
            my $out_i = "$out.$i";
            foreach my $r (sort criterion keys %{$Href}) {
               my $ref = $Href->{$r};
               my $ref_i = "$ref.$i";
               push(@cfgline, $ref_i);
            }
            if (!(-e $out_i) or $remakeREPORTS) {
               open(OUTi, "> $out_i") or die "couldn't open file: $out_i\n";
               $auxOUT =~ s/^ +//g; $auxOUT =~ s/ +$//g;
               if (($auxOUT eq "") or ($auxOUT =~ /^[\?\!\.]$/)) { $auxOUT = $EMPTY_ITEM; }
               print OUTi $auxOUT, "\n";
               close(OUTi);
            }
            if ($verbose > 1) { 
               if ($i % 10 == 0) { print STDERR "."; }
               if ($i % 100 == 0) { print STDERR "$i"; }
	    }
            $i++;
            print CFG "$out_i ", join(" ", @cfgline), "\n";
  	 }
         if ($verbose > 1) { print STDERR "..", $i-1, " segments [DONE]\n"; }
         close(CFG);
         close(OUT);

         if ($verbose > 1) { print STDERR "building $reportROUGE...\n"; }
         system("$toolROUGE -m $configROUGE > $reportROUGE");  # always with stemming
         system("rm $configROUGE");
         my $j = 1;
         while ($j <= $i) { if (-e "$out.$j") { system("rm $out.$j"); } $j++; }
         foreach my $r (sort criterion keys %{$Href}) {
            my $ref = $Href->{$r};
            $j = 1;
            while ($j <= $i) { if (-e "$ref.$j") { system("rm $ref.$j"); } $j++; }
         }

         my ($ROUGE1, $ROUGE2, $ROUGE3, $ROUGE4, $ROUGEL, $ROUGES, $ROUGESU, $ROUGEW) = read_rouge($reportROUGE);
         my ($rouge1, $rouge2, $rouge3, $rouge4, $rougeL, $rougeS, $rougeSU, $rougeW) = read_rouge_segments($reportROUGE);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-1", $ROUGE1, $rouge1, $reportROUGE1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-2", $ROUGE2, $rouge2, $reportROUGE2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-3", $ROUGE3, $rouge3, $reportROUGE3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-4", $ROUGE4, $rouge4, $reportROUGE4xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-L", $ROUGEL, $rougeL, $reportROUGELxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-Ss", $ROUGES, $rougeS, $reportROUGESxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-SUs", $ROUGESU, $rougeSU, $reportROUGESUxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-W-1.2", $ROUGEW, $rougeW, $reportROUGEWxml, $verbose);
         system("rm $reportROUGE");
      }
   }
}

sub doMultiWER {
   #description _ computes 1-WER score (multiple reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [list reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"1-WER"}) {

      if ($verbose == 1) { print STDERR "WER.."; }

      my $reportWERxml = "$fname.1-$WEREXT.$XMLEXT";
      my $reportWERxmlgz = "$fname.1-$WEREXT.$XMLEXT.$GZEXT";

      if (((!(-e $reportWERxml) and !(-e $reportWERxmlgz)) or $remakeREPORTS) and $m->{"1-WER"}) {

         my $toolWER = "perl $tools/$METRICS::TWER";

         my $refNISTsgml = $REFname.".$REFEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_multidoc($Href, $refNISTsgml, 2, $verbose); }
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }

         my $reportWER = "$fname.$WEREXT.$REPORTEXT";
         if ($verbose > 1) { print STDERR "building $reportWER...\n"; }
         system("$toolWER -s $srcNISTsgml -t $outNISTsgml -r $refNISTsgml > $reportWER");
         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         my $WER = read_WER($reportWER);
         my $wer = read_WER_segments($reportWER);
         IQXML::write_report($TGTname, $REFname, "1-WER", $WER, $wer, $reportWERxml, $verbose);
         system("rm $reportWER");
      }
   }
}

sub doMultiPER {
   #description _ computes 1-PER score (multiple reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [list reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"1-PER"}) {

      if ($verbose == 1) { print STDERR "PER.."; }

      my $reportPERxml = "$fname.1-$PEREXT.$XMLEXT";
      my $reportPERxmlgz = "$fname.1-$PEREXT.$XMLEXT.$GZEXT";

      if (((!(-e $reportPERxml) and (!(-e $reportPERxmlgz))) or $remakeREPORTS) and $m->{"1-PER"}) {

         my $toolPER = "perl $tools/$METRICS::TPER";

         my $refNISTsgml = $REFname.".$REFEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_multidoc($Href, $refNISTsgml, 2, $verbose); }
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }
         my $reportPER = "$fname.$PEREXT.$REPORTEXT";
         if ($verbose > 1) { print STDERR "building $reportPER...\n"; }
         system("$toolPER -s $srcNISTsgml -t $outNISTsgml -r $refNISTsgml > $reportPER");
         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         my $PER = read_PER($reportPER);
         my $per = read_PER_segments($reportPER);
         IQXML::write_report($TGTname, $REFname, "1-PER", $PER, $per, $reportPERxml, $verbose);
         system("rm $reportPER");
      }
   }
}

# *************************************************************************************
# ************************* INDIVIDUAL METRICS FOR QARLA (single reference) ***********
# *************************************************************************************

sub doBLEU {
   #description _ computes BLEU score (by calling NIST mteval script) -> n = 1..4 (single reference)
   #param1 _ source file
   #param2 _ reference file 
   #param3 _ candidate file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ target NAME (base filename)
   #param6 _ reference NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $ref = shift;
   my $out = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"$BLEUEXT-1"} or $m->{"$BLEUEXT-2"} or $m->{"$BLEUEXT-3"} or $m->{"$BLEUEXT-4"}) {

      if ($verbose == 1) { print STDERR "BLEU.."; }

      my $reportBLEU1xml = "$fname.$BLEUEXT-1.$XMLEXT";
      my $reportBLEU2xml = "$fname.$BLEUEXT-2.$XMLEXT";
      my $reportBLEU3xml = "$fname.$BLEUEXT-3.$XMLEXT";
      my $reportBLEU4xml = "$fname.$BLEUEXT-4.$XMLEXT";
      my $reportBLEU1xmlgz = "$fname.$BLEUEXT-1.$XMLEXT.$GZEXT";
      my $reportBLEU2xmlgz = "$fname.$BLEUEXT-2.$XMLEXT.$GZEXT";
      my $reportBLEU3xmlgz = "$fname.$BLEUEXT-3.$XMLEXT.$GZEXT";
      my $reportBLEU4xmlgz = "$fname.$BLEUEXT-4.$XMLEXT.$GZEXT";

      if ((!(-e $reportBLEU1xml) and !(-e $reportBLEU1xmlgz)) or (!(-e $reportBLEU2xml) and !(-e $reportBLEU2xmlgz)) or (!(-e $reportBLEU3xml) and !(-e $reportBLEU3xmlgz)) or (!(-e $reportBLEU4xml) and !(-e $reportBLEU4xmlgz)) or $remakeREPORTS) {

         my $toolBLEU = "perl $tools/$METRICS::TBLEU";

         my $refNISTsgml = "$ref.$REFEXT.$NISTEXT.$SGMLEXT";
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($ref, $refNISTsgml, 2, $verbose); }
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }
         my $reportBLEU = "$fname.$BLEUEXT.$REPORTEXT";
         if ($verbose > 1) { print STDERR "building $reportBLEU...\n"; }
         system("$toolBLEU -s $srcNISTsgml -t $outNISTsgml -r $refNISTsgml > $reportBLEU");
         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         my ($BLEU1, $BLEU2, $BLEU3, $BLEU4) = read_bleu($reportBLEU);
         my ($bleu1, $bleu2, $bleu3, $bleu4) = read_bleu_segments($reportBLEU);
         system("rm $reportBLEU");
         IQXML::write_report($TGTname, $REFname, "$BLEUEXT-1", $BLEU1, $bleu1, $reportBLEU1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$BLEUEXT-2", $BLEU2, $bleu2, $reportBLEU2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$BLEUEXT-3", $BLEU3, $bleu3, $reportBLEU3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$BLEUEXT-4", $BLEU4, $bleu4, $reportBLEU4xml, $verbose);
      }
   }
}

sub doGTM {
   #description _ computes GTM F1 (by calling Proteus java gtm) -> e = 1..3 (single reference)
   #param1 _ source file
   #param2 _ reference file 
   #param3 _ candidate file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ target NAME (base filename)
   #param6 _ reference NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $ref = shift;
   my $out = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"$GTMEXT-1"} or $m->{"$GTMEXT-2"} or $m->{"$GTMEXT-3"}) {

      if ($verbose == 1) { print STDERR "GTM.."; }

      my $reportGTM1xml = "$fname.$GTMEXT-1.$XMLEXT";
      my $reportGTM2xml = "$fname.$GTMEXT-2.$XMLEXT";
      my $reportGTM3xml = "$fname.$GTMEXT-3.$XMLEXT";
      my $reportGTM1xmlgz = "$fname.$GTMEXT-1.$XMLEXT.$GZEXT";
      my $reportGTM2xmlgz = "$fname.$GTMEXT-2.$XMLEXT.$GZEXT";
      my $reportGTM3xmlgz = "$fname.$GTMEXT-3.$XMLEXT.$GZEXT";

      if ((!(-e $reportGTM1xml) and !(-e $reportGTM1xmlgz)) or (!(-e $reportGTM2xml) and !(-e $reportGTM2xmlgz)) or (!(-e $reportGTM3xml) and !(-e $reportGTM3xmlgz)) or $remakeREPORTS) {

         my $toolGTM = "java -jar $tools/$METRICS::TGTM";

         my $refGTMsgml = "$ref.$GTMEXT.$SGMLEXT";
         my $outGTMsgml = "$out.$GTMEXT.$SGMLEXT";
         if ((!(-e $refGTMsgml)) or $remakeREPORTS) { IQXML::GTM_f_create_mteval_doc($ref, $refGTMsgml, $verbose); }
         if ((!(-e $outGTMsgml)) or $remakeREPORTS) { IQXML::GTM_f_create_mteval_doc($out, $outGTMsgml, $verbose); }

         my $reportGTM1 = "$fname.$GTMEXT-1.$REPORTEXT";
         my $reportGTM2 = "$fname.$GTMEXT-2.$REPORTEXT";
         my $reportGTM3 = "$fname.$GTMEXT-3.$REPORTEXT";

         if (((!(-e $reportGTM1xml) and !(-e $reportGTM1xmlgz)) or $remakeREPORTS) and $m->{"$GTMEXT-1"}) {
            if ($verbose > 1) { print STDERR "building $reportGTM1...\n"; }
            system("$toolGTM -e 1 $outGTMsgml $refGTMsgml > $reportGTM1");
            my $GTM1 = read_gtm($reportGTM1);
            my $gtm1 = read_gtm_segments($reportGTM1);
            IQXML::write_report($TGTname, $REFname, "$GTMEXT-1", $GTM1, $gtm1, $reportGTM1xml, $verbose);
            system("rm $reportGTM1");
         }
         if (((!(-e $reportGTM2xml) and !(-e $reportGTM2xmlgz)) or $remakeREPORTS) and $m->{"$GTMEXT-2"}) {
            if ($verbose > 1) { print STDERR "building $reportGTM2...\n"; }
            system("$toolGTM -e 2 $outGTMsgml $refGTMsgml > $reportGTM2");
            my $GTM2 = read_gtm($reportGTM2);
            my $gtm2 = read_gtm_segments($reportGTM2);
            IQXML::write_report($TGTname, $REFname, "$GTMEXT-2", $GTM2, $gtm2, $reportGTM2xml, $verbose);
            system("rm $reportGTM2");
         }
         if (((!(-e $reportGTM3xml) and !(-e $reportGTM3xmlgz)) or $remakeREPORTS) and $m->{"$GTMEXT-3"}) {
            if ($verbose > 1) { print STDERR "building $reportGTM3...\n"; }
            system("$toolGTM -e 3 $outGTMsgml $refGTMsgml > $reportGTM3");
            my $GTM3 = read_gtm($reportGTM3);
            my $gtm3 = read_gtm_segments($reportGTM3);
            IQXML::write_report($TGTname, $REFname, "$GTMEXT-3", $GTM3, $gtm3, $reportGTM3xml, $verbose);
            system("rm $reportGTM3");
         }

         if (-e $refGTMsgml) { system "rm $refGTMsgml"; }
         if (-e $outGTMsgml) { system "rm $outGTMsgml"; }
      }
   }
}

sub doMETEOR {
   #description _ computes METEOR scores (exact + stem + wnstem + wnsyn) (single reference)
   #param1 _ source file
   #param2 _ reference file 
   #param3 _ candidate file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ target NAME (base filename)
   #param6 _ reference NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $ref = shift;
   my $out = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"$METEXT-exact"} or $m->{"$METEXT-stem"} or $m->{"$METEXT-wnstm"} or $m->{"$METEXT-wnsyn"}) {

      if ($verbose == 1) { print STDERR "METEOR.."; }

      my $reportMETexactXML = "$fname.$METEXT-exact.$XMLEXT";
      my $reportMETstemXML = "$fname.$METEXT-stem.$XMLEXT";
      my $reportMETwnstmXML = "$fname.$METEXT-wnstm.$XMLEXT";
      my $reportMETwnsynXML = "$fname.$METEXT-wnsyn.$XMLEXT";
      my $reportMETexactXMLgz = "$fname.$METEXT-exact.$XMLEXT.$GZEXT";
      my $reportMETstemXMLgz = "$fname.$METEXT-stem.$XMLEXT.$GZEXT";
      my $reportMETwnstmXMLgz = "$fname.$METEXT-wnstm.$XMLEXT.$GZEXT";
      my $reportMETwnsynXMLgz = "$fname.$METEXT-wnsyn.$XMLEXT.$GZEXT";

      if ((!(-e $reportMETexactXML) and !(-e $reportMETexactXMLgz)) or (!(-e $reportMETstemXML) and !(-e $reportMETstemXMLgz)) or (!(-e $reportMETwnstmXML) and !(-e $reportMETwnstmXMLgz)) or (!(-e $reportMETwnsynXML) and !(-e $reportMETwnsynXMLgz)) or $remakeREPORTS) {

         my $toolMETEOR = "perl $tools/$METRICS::TMETEOR";

         my $refNISTsgml = "$ref.$REFEXT.$NISTEXT.$SGMLEXT";
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($ref, $refNISTsgml, 2, $verbose); }
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }
         my $refMETsgml = "$ref.$REFEXT.$METEXT.$SGMLEXT";
         if ((!(-e $refMETsgml)) or $remakeREPORTS) {
            system "awk '{if (\$0 ~ /seg id=\"/) { gsub(/id=\"/,\"id=\"); gsub(/\">/,\">\"); } print;}' < $refNISTsgml > $refMETsgml";
         }
         my $outMETsgml = "$out.$METEXT.$SGMLEXT";
         if ((!(-e $outMETsgml)) or $remakeREPORTS) {
            system "awk '{if (\$0 ~ /seg id=\"/) { gsub(/id=\"/,\"id=\"); gsub(/\">/,\">\"); } print;}' < $outNISTsgml > $outMETsgml";
         }

         my $reportMETexact = "$fname.$METEXT-exact.$REPORTEXT";
         my $reportMETstem = "$fname.$METEXT-stem.$REPORTEXT";
         my $reportMETwnstm = "$fname.$METEXT-wnstm.$REPORTEXT";
         my $reportMETwnsyn = "$fname.$METEXT-wnsyn.$REPORTEXT";

         if (((!(-e $reportMETexactXML) and !(-e $reportMETexactXMLgz)) or $remakeREPORTS) and $m->{"$METEXT-exact"}) { #exact
            if ($verbose > 1) { print STDERR "building $reportMETexact ...\n"; }
            system("$toolMETEOR -s dummysysTST -t $outMETsgml -r $refMETsgml --modules \"exact\" > $reportMETexact");
            my $METEOREXACT = read_meteor($reportMETexact);
            my $meteorexact = read_meteor_segments($reportMETexact);
            IQXML::write_report($TGTname, $REFname, "$METEXT-exact", $METEOREXACT, $meteorexact, $reportMETexactXML, $verbose);
            system("rm $reportMETexact");
         }
         if (((!(-e $reportMETstemXML) and !(-e $reportMETstemXMLgz)) or $remakeREPORTS) and $m->{"$METEXT-stem"}) { #exact + porter
            if ($verbose > 1) { print STDERR "building $reportMETstem...\n"; }
            system("$toolMETEOR -s dummysysTST -t $outMETsgml -r $refMETsgml > $reportMETstem");
            my $METEORSTEM = read_meteor($reportMETstem);
            my $meteorstem = read_meteor_segments($reportMETstem);
            IQXML::write_report($TGTname, $REFname, "$METEXT-stem", $METEORSTEM, $meteorstem, $reportMETstemXML, $verbose);
            system("rm $reportMETstem");
         }
         if (((!(-e $reportMETwnstmXML) and !(-e $reportMETwnstmXMLgz)) or $remakeREPORTS) and $m->{"$METEXT-wnstm"}) { #exact + porter_stem + wn_stem
            if ($verbose > 1) { print STDERR "building $reportMETwnstm ...\n"; }
            system("$toolMETEOR -s dummysysTST -t $outMETsgml -r $refMETsgml --modules \"exact porter_stem wn_stem\"  > $reportMETwnstm");
            my $METEORWNSTM = read_meteor($reportMETwnstm);
            my $meteorwnstm = read_meteor_segments($reportMETwnstm);
            IQXML::write_report($TGTname, $REFname, "$METEXT-wnstm", $METEORWNSTM, $meteorwnstm, $reportMETwnstmXML, $verbose);
            system("rm $reportMETwnstm");
         }
         if (((!(-e $reportMETwnsynXML) and !(-e $reportMETwnsynXMLgz)) or $remakeREPORTS) and $m->{"$METEXT-wnsyn"}) { #exact + porter_stem + wn_stem + wn_syn
            if ($verbose > 1) { print STDERR "building $reportMETwnsyn ...\n"; }
            system("$toolMETEOR -s dummysysTST -t $outMETsgml -r $refMETsgml --modules \"exact porter_stem wn_stem wn_synonymy\" > $reportMETwnsyn");
            my $METEORWNSYN = read_meteor($reportMETwnsyn);
            my $meteorwnsyn = read_meteor_segments($reportMETwnsyn);
            IQXML::write_report($TGTname, $REFname, "$METEXT-wnsyn", $METEORWNSYN, $meteorwnsyn, $reportMETwnsynXML, $verbose);
            system("rm $reportMETwnsyn");
         }

         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         if (-e $refMETsgml) { system "rm $refMETsgml"; }
         if (-e $outMETsgml) { system "rm $outMETsgml"; }
      }
   }
}

sub doNIST {
   #description _ computes NIST score (by calling NIST mteval script) -> n = 1..5 (single reference)
   #param1 _ source file
   #param2 _ reference file 
   #param3 _ candidate file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ target NAME (base filename)
   #param6 _ reference NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $ref = shift;
   my $out = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"$NISTEXT-1"} or $m->{"$NISTEXT-2"} or $m->{"$NISTEXT-3"} or $m->{"$NISTEXT-4"} or $m->{"$NISTEXT-5"}) {

      if ($verbose == 1) { print STDERR "NIST.."; }

      my $reportNIST1xml = "$fname.$NISTEXT-1.$XMLEXT";
      my $reportNIST2xml = "$fname.$NISTEXT-2.$XMLEXT";
      my $reportNIST3xml = "$fname.$NISTEXT-3.$XMLEXT";
      my $reportNIST4xml = "$fname.$NISTEXT-4.$XMLEXT";
      my $reportNIST5xml = "$fname.$NISTEXT-5.$XMLEXT";
      my $reportNIST1xmlgz = "$fname.$NISTEXT-1.$XMLEXT.$GZEXT";
      my $reportNIST2xmlgz = "$fname.$NISTEXT-2.$XMLEXT.$GZEXT";
      my $reportNIST3xmlgz = "$fname.$NISTEXT-3.$XMLEXT.$GZEXT";
      my $reportNIST4xmlgz = "$fname.$NISTEXT-4.$XMLEXT.$GZEXT";
      my $reportNIST5xmlgz = "$fname.$NISTEXT-5.$XMLEXT.$GZEXT";

      if ((!(-e $reportNIST1xml) and !(-e $reportNIST1xmlgz)) or (!(-e $reportNIST2xml) and !(-e $reportNIST2xmlgz)) or (!(-e $reportNIST3xml) and !(-e $reportNIST3xmlgz)) or (!(-e $reportNIST4xml) and !(-e $reportNIST4xmlgz)) or (!(-e $reportNIST5xml) and !(-e $reportNIST5xmlgz)) or $remakeREPORTS) {

         my $toolNIST = "perl $tools/$METRICS::TNIST";
         my $refNISTsgml = "$ref.$REFEXT.$NISTEXT.$SGMLEXT";
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($ref, $refNISTsgml, 2, $verbose); }
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }

         my $reportNIST = "$fname.$NISTEXT.$REPORTEXT";
         if ($verbose > 1) { print STDERR "building $reportNIST...\n"; }
         system("$toolNIST -s $srcNISTsgml -t $outNISTsgml -r $refNISTsgml > $reportNIST");
         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         my ($NIST1, $NIST2, $NIST3, $NIST4, $NIST5) = read_nist($reportNIST);
         my ($nist1, $nist2, $nist3, $nist4, $nist5) = read_nist_segments($reportNIST);
         system("rm $reportNIST");
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-1", $NIST1, $nist1, $reportNIST1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-2", $NIST2, $nist2, $reportNIST2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-3", $NIST3, $nist3, $reportNIST3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-4", $NIST4, $nist4, $reportNIST4xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$NISTEXT-5", $NIST5, $nist5, $reportNIST5xml, $verbose);
      }
   }
}

sub doROUGE {
   #description _ computes ROUGE scores -> n = 1..4, LCS, S*, SU*, W-1.2 (single reference)
   #param1 _ source file
   #param2 _ reference file 
   #param3 _ candidate file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ target NAME (base filename)
   #param6 _ reference NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $ref = shift;
   my $out = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   my $reportROUGE1xml = "$fname.$ROUGEXT-1.$XMLEXT";
   my $reportROUGE2xml = "$fname.$ROUGEXT-2.$XMLEXT";
   my $reportROUGE3xml = "$fname.$ROUGEXT-3.$XMLEXT";
   my $reportROUGE4xml = "$fname.$ROUGEXT-4.$XMLEXT";
   my $reportROUGELxml = "$fname.$ROUGEXT-L.$XMLEXT";
   my $reportROUGESxml = "$fname.$ROUGEXT-Ss.$XMLEXT";
   my $reportROUGESUxml = "$fname.$ROUGEXT-SUs.$XMLEXT";
   my $reportROUGEWxml = "$fname.$ROUGEXT-W-1.2.$XMLEXT";
   my $reportROUGE1xmlgz = "$fname.$ROUGEXT-1.$XMLEXT.$GZEXT";
   my $reportROUGE2xmlgz = "$fname.$ROUGEXT-2.$XMLEXT.$GZEXT";
   my $reportROUGE3xmlgz = "$fname.$ROUGEXT-3.$XMLEXT.$GZEXT";
   my $reportROUGE4xmlgz = "$fname.$ROUGEXT-4.$XMLEXT.$GZEXT";
   my $reportROUGELxmlgz = "$fname.$ROUGEXT-L.$XMLEXT.$GZEXT";
   my $reportROUGESxmlgz = "$fname.$ROUGEXT-Ss.$XMLEXT.$GZEXT";
   my $reportROUGESUxmlgz = "$fname.$ROUGEXT-SUs.$XMLEXT.$GZEXT";
   my $reportROUGEWxmlgz = "$fname.$ROUGEXT-W-1.2.$XMLEXT.$GZEXT";

   if ($m->{"$ROUGEXT-1"} or $m->{"$ROUGEXT-2"} or $m->{"$ROUGEXT-3"} or $m->{"$ROUGEXT-4"} or $m->{"$ROUGEXT-L"} or $m->{"$ROUGEXT-Ss"} or $m->{"$ROUGEXT-SUs"} or $m->{"$ROUGEXT-W-1.2"}) {

      if ($verbose == 1) { print STDERR "ROUGE.."; }

      if ((!(-e $reportROUGE1xml) and !(-e $reportROUGE1xmlgz)) or (!(-e $reportROUGE2xml) and !(-e $reportROUGE2xmlgz)) or (!(-e $reportROUGE3xml) and !(-e $reportROUGE3xmlgz)) or (!(-e $reportROUGE4xml) and !(-e $reportROUGE4xmlgz)) or (!(-e $reportROUGELxml) and !(-e $reportROUGELxmlgz)) or (!(-e $reportROUGESxml) and !(-e $reportROUGESxmlgz)) or (!(-e $reportROUGESUxml) and !(-e $reportROUGESUxmlgz)) or (!(-e $reportROUGEWxml) and !(-e $reportROUGEWxmlgz)) or $remakeREPORTS) {

         my $toolROUGE = "$tools/$METRICS::TROUGE -e $tools/$METRICS::TROUGEdata -z SPL -2 -1 -U -r 1000 -n 4 -w 1.2 -c 95 -d"; #always with stemming

         my $configROUGE = "$fname.$ROUGEXT.$CFGEXT";
         my $reportROUGE = "$fname.$ROUGEXT.$REPORTEXT";

         open(OUT, "< $out") or die "couldn't open file: $out\n";
         open(REF, "< $ref") or die "couldn't open file: $ref\n";
         open(CFG, "> $configROUGE") or die "couldn't open file: $configROUGE\n";
         my $i = 1;
         while (my $auxREF = <REF>) {
            my $auxOUT = <OUT>;
            chomp($auxREF);
            chomp($auxOUT);
            my $out_i = "$out.$i";
            my $ref_i = "$ref.$i";
            if (!(-e $ref_i) or $remakeREPORTS) {
               open(REFi, "> $ref_i") or die "couldn't open file: $ref_i\n";
               $auxREF =~ s/^ +//g; $auxREF =~ s/ +$//g;
               if (($auxREF eq "") or ($auxREF =~ /^[\?\!\.]$/)){ $auxREF = $EMPTY_ITEM; }
               print REFi $auxREF, "\n";
               close(REFi);
            }
            if (!(-e $out_i) or $remakeREPORTS) {
               open(OUTi, "> $out_i") or die "couldn't open file: $out_i\n";
               $auxOUT =~ s/^ +//g; $auxOUT =~ s/ +$//g;
               if (($auxOUT eq "") or ($auxOUT =~ /^[\?\!\.]$/)) { $auxOUT = $EMPTY_ITEM; }
               print OUTi $auxOUT, "\n";
               close(OUTi);
            }
            print CFG "$out_i $ref_i\n";
            $i++;
         }
         close(CFG);
         close(REF);
         close(OUT);
         if ($verbose > 1) { print STDERR "building $reportROUGE...\n"; }
         system("$toolROUGE -m $configROUGE > $reportROUGE");
         system("rm $configROUGE");
         my $j = 1;
         while ($j <= $i) { if ((-e "$ref.$j")) { system("rm $ref.$j"); } $j++; }
         $j = 1;
         while ($j <= $i) { if ((-e "$out.$j")) { system("rm $out.$j"); } $j++; }
         my ($ROUGE1, $ROUGE2, $ROUGE3, $ROUGE4, $ROUGEL, $ROUGES, $ROUGESU, $ROUGEW) = read_rouge($reportROUGE);
         my ($rouge1, $rouge2, $rouge3, $rouge4, $rougeL, $rougeS, $rougeSU, $rougeW) = read_rouge_segments($reportROUGE);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-1", $ROUGE1, $rouge1, $reportROUGE1xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-2", $ROUGE2, $rouge2, $reportROUGE2xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-3", $ROUGE3, $rouge3, $reportROUGE3xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-4", $ROUGE4, $rouge4, $reportROUGE4xml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-L", $ROUGEL, $rougeL, $reportROUGELxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-Ss", $ROUGES, $rougeS, $reportROUGESxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-SUs", $ROUGESU, $rougeSU, $reportROUGESUxml, $verbose);
         IQXML::write_report($TGTname, $REFname, "$ROUGEXT-W-1.2", $ROUGEW, $rougeW, $reportROUGEWxml, $verbose);
         system("rm $reportROUGE");
      }
   }
}

sub doWER {
   #description _ computes 1-WER score (single reference)
   #param1 _ source file
   #param2 _ reference file 
   #param3 _ candidate file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ target NAME (base filename)
   #param6 _ reference NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $ref = shift;
   my $out = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"1-WER"}) {

      if ($verbose == 1) { print STDERR "WER.."; }

      my $reportWERxml = "$fname.1-$WEREXT.$XMLEXT";
      my $reportWERxmlgz = "$fname.1-$WEREXT.$XMLEXT.$GZEXT";

      if (((!(-e $reportWERxml) and !(-e $reportWERxmlgz)) or $remakeREPORTS) and $m->{"1-WER"}) {
         my $toolWER = "perl $tools/$METRICS::TWER";

         my $refNISTsgml = "$ref.$REFEXT.$NISTEXT.$SGMLEXT";
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($ref, $refNISTsgml, 2, $verbose); }
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }

         my $reportWER = "$fname.$WEREXT.$REPORTEXT";
         if ($verbose > 1) { print STDERR "building $reportWER...\n"; }
         system("$toolWER -s $srcNISTsgml -t $outNISTsgml -r $refNISTsgml > $reportWER");
         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         my $WER = read_WER($reportWER);
         my $wer = read_WER_segments($reportWER);
         IQXML::write_report($TGTname, $REFname, "1-WER", $WER, $wer, $reportWERxml, $verbose);
         system("rm $reportWER");
      }
   }
}

sub doPER {
   #description _ computes 1-PER score (single reference)
   #param1 _ source file
   #param2 _ reference file 
   #param3 _ candidate file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ target NAME (base filename)
   #param6 _ reference NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $ref = shift;
   my $out = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $m = shift;
   my $verbose = shift;

   my $fname = $TGTname."-".$REFname;

   if ($m->{"1-PER"}) {

      if ($verbose == 1) { print STDERR "PER.."; }

      my $reportPERxml = "$fname.1-$PEREXT.$XMLEXT";
      my $reportPERxmlgz = "$fname.1-$PEREXT.$XMLEXT.$GZEXT";

      if (((!(-e $reportPERxml) and (!(-e $reportPERxmlgz))) or $remakeREPORTS) and $m->{"1-PER"}) {
         my $toolPER = "perl $tools/$METRICS::TPER";

         my $refNISTsgml = "$ref.$REFEXT.$NISTEXT.$SGMLEXT";
         my $srcNISTsgml = "$src.$SRCEXT.$NISTEXT.$SGMLEXT";
         if ((!(-e $refNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($ref, $refNISTsgml, 2, $verbose); }
         if ((!(-e $srcNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($src, $srcNISTsgml, 0, $verbose); }
         my $outNISTsgml = "$out.$NISTEXT.$SGMLEXT";
         if ((!(-e $outNISTsgml)) or $remakeREPORTS) { IQXML::NIST_f_create_mteval_doc($out, $outNISTsgml, 1, $verbose); }
         my $reportPER = "$fname.$PEREXT.$REPORTEXT";
         if ($verbose > 1) { print STDERR "building $reportPER...\n"; }
         system("$toolPER -s $srcNISTsgml -t $outNISTsgml -r $refNISTsgml > $reportPER");
         if (-e $srcNISTsgml) { system "rm $srcNISTsgml"; }
         if (-e $refNISTsgml) { system "rm $refNISTsgml"; }
         if (-e $outNISTsgml) { system "rm $outNISTsgml"; }
         my $PER = read_PER($reportPER);
         my $per = read_PER_segments($reportPER);
         IQXML::write_report($TGTname, $REFname, "1-PER", $PER, $per, $reportPERxml, $verbose);
         system("rm $reportPER");
      }
   }
}

# *************************************************************************************
# ************************* DO METRICS ************************************************
# *************************************************************************************

sub doMetrics {
   #description _ MLT evaluation: * generates SGML evaluation files
   #                              * computes GTM f1 (by calling Proteus java gtm) -> e = 1..3
   #                              * computes BLEU score (by calling NIST mteval script) -> n = 4
   #                              * computes NIST score (by calling NIST mteval script) -> n = 5
   #                              * computes METEOR score
   #                              * computes WER
   #                              * computes PER
   #                              * computes ROUGE
   #param1 _ source file 
   #param2 _ candidate file
   #param3 _ reference file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ target NAME (base filename)
   #param6 _ reference NAME (base filename)
   #param7 _ TOOL directory
   #param8 _ set of metrics
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $ref = shift;
   my $remakeREPORTS = shift;
   my $TGT = shift;
   my $REF = shift;
   my $tools = shift;
   my $M = shift;
   my $verbose = shift;

   if ($verbose > 1) { print STDERR "computing MT-scores [$TGT] vs [$REF]...\n"; }
   elsif ($verbose == 1) { print STDERR "$TGT-$REF ["; }

   doBLEU($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doGTM($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doMETEOR($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doNIST($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doROUGE($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doWER($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doPER($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);

   if ($verbose == 1) { print STDERR "]\n"; }
}

sub doMultiMetrics {
   #description _ MLT evaluation: for multiple reference
   #                              * generates SGML evaluation files
   #                              * computes GTM f1 (by calling Proteus java gtm) -> e = 1..3
   #                              * computes BLEU score (by calling NIST mteval script) -> n = 4
   #                              * computes NIST score (by calling NIST mteval script) -> n = 5
   #                              * computes METEOR score
   #                              * computes WER
   #                              * computes PER
   #                              * computes ROUGE
   #param1 _ source file 
   #param2 _ candidate file
   #param3 _ reference file(s) (list ref)
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ SYSTEM NAME (base filename)
   #param6 _ TOOL directory
   #param7 _ set of metrics
   #param8 _ level of granularity ('sys' for system; 'seg' for segment)
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGT = shift;
   my $tools = shift;
   my $M = shift;
   my $verbose = shift;

   my $REF = join("_", sort criterion keys %{$Href});

   if ($verbose > 1) { print STDERR "computing MT-scores [$TGT]...\n"; }
   elsif ($verbose == 1) { print STDERR "$TGT-$REF ["; }

   doMultiBLEU($src, $out, $Href, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doMultiGTM($src, $out, $Href, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doMultiMETEOR($src, $out, $Href, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doMultiNIST($src, $out, $Href, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doMultiROUGE($src, $out, $Href, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doMultiWER($src, $out, $Href, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   doMultiPER($src, $out, $Href, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);

   if ($verbose == 1) { print STDERR "]\n"; }
}

# *************************************************************************************
# ************************* METRIC REPORTS ********************************************
# *************************************************************************************

sub read_metrics {
    #description _ read an xml IQREPORT file and print it onto a given file
    #param1  _ set of metrics
    #param2  _ set of systems
    #param3  _ set of references
    #param4  _ set of segments (topics)
    #param5  _ hash of scores (outside QARLA)
    #param6  _ level of granularity ('sys' for system; 'seg' for segment)
    #param7  _ verbosity (0/1)

    my $M = shift;
    my $S = shift;
    my $R = shift;
    my $T = shift;
    my $hOQ = shift;
    my $G = shift;
    my $verbose = shift;

    my @topics;
    foreach my $t (@{$T}) { $topics[$t] = 1; }
    
    foreach my $s (sort criterion @{$S}) {
       foreach my $m (sort @{$M}) {
          IQXML::read_report($s, join("_", sort criterion @{$R}), $m, "$s-".join("_", sort criterion @{$R}).".$m.$METRICS::XMLEXT", $hOQ, \@topics, $G, $verbose);
       }
    }
}

sub read_write_metrics {
    #description _ read an xml IQREPORT file and print it onto a given file
    #param1 _ filehandle
    #param2 _ target
    #param3 _ reference
    #param4 _ metrics
    #param5 _ verbosity (0/1)

    my $file = shift;
    my $T = shift;
    my $R = shift;
    my $M = shift;
    my $verbose = shift;

    foreach my $m (sort keys %{$M}) {
       IQXML::read_write_report($T, $R, $m, "$T-$R.$m.$METRICS::XMLEXT", $file, $verbose);
    }
}

sub print_metrics {
    #description _ printing of metric scores inside/outside QARLA onto stdout
    #param1  _ hash of scores (outside QARLA)
    #param2  _ hash of scores (inside QARLA)
    #param3  _ system name
    #param4  _ level of granularity ('sys' for system; 'seg' for segment)
    #param5  _ metric set
    #param6  _ set of segments (topics)
    #param7  _ set of references (models)
    #param8  _ do metrics outside QARLA (0/1)
    #param9  _ verbosity (0/1)

    my $hOQ = shift;
    my $hIQ = shift;
    my $system = shift;
    my $G = shift;
    my $M = shift;
    my $T = shift;
    my $R = shift;
    my $doOQ = shift;
    my $verbose = shift;

    my $ref = join("_", sort criterion @{$R});

    if ($G eq 'sys') { # G == 'sys'
       my @scores;
       if ($doOQ) {
          foreach my $m (sort @{$M}) { push(@scores, sprintf("%8.4f", $hOQ->{$m}->{$system}->{$ref})); }
       }
       my @auxSYS = ($system);
       my $QUEEN = QARLA::QUEEN($hIQ, $M, $T, \@auxSYS, $R, $verbose);
       my $IQ = QARLA::IQ($hIQ, $M, $T, \@auxSYS, $R, $verbose);
       printf "%-6s %s %8.4f %8.4f\n", $system, join(" ", @scores), $QUEEN, $IQ;
    }
    else { # G == 'seg'
       my $nSEGMENTS = scalar(@{$hIQ->{$M->[0]}});
       my $segment = 1;
       while ($segment < $nSEGMENTS) {
          my @scores;
          if ($doOQ) {
             foreach my $m (sort @{$M}) { push(@scores, sprintf("%8.4f", $hOQ->{$m}->[$segment]->{$system}->{$ref})); }
          }
          my @auxSYS = ($system); my @auxSEG = ($segment);
          my $QUEEN = QARLA::QUEEN($hIQ, $M, \@auxSEG, \@auxSYS, $R, $verbose);
          my $IQ = QARLA::IQ($hIQ, $M, \@auxSEG, \@auxSYS, $R, $verbose);
          printf "%-14s %s %8.4f %8.4f\n", $system.":".$segment, join(" ", @scores), $QUEEN, $IQ;
          $segment++;
       }
    }
}

sub print_rankings
{
    #description _ printing of ranking lists of system scores on a metric-by-metric basis
    #param1  _ metrics outside QARLA (hash ref)
    #param2  _ metrics inside QARLA (hash ref)
    #param3  _ level of granularity ('sys'/'seg')
    #param4  _ metric set
    #param5  _ set of segments (topics)
    #param6  _ set of systems
    #param7  _ set of references (models)
    #param8  _ do metrics outside QARLA (0/1)
    #param9  _ verbosity (0/1)

    my $hOQ = shift;
    my $hIQ = shift;
    my $G = shift;
    my $M = shift;
    my $T = shift;
    my $S = shift;
    my $R = shift;
    my $doOQ = shift;
    my $verbose = shift;

    my $ref = join("_", sort criterion @{$R});

    if ($G eq 'sys') { # G == 'sys'
       if ($doOQ) {
          foreach my $m (sort @{$M}) {
             print "RANK=$m\n";
             foreach my $system (sort criterion @{$S}) {
                printf("%-6s %8.4f\n", $system, $hOQ->{$m}->{$system}->{$ref});
	     }
	  }
       }
       print "RANK=QUEEN\n";
       foreach my $system (sort criterion @{$S}) {
          my @auxSYS = ($system);
          my $QUEEN = QARLA::QUEEN($hIQ, $M, $T, \@auxSYS, $R, $verbose);
          printf("%-6s %8.4f\n", $system, $QUEEN);
       }
       print "RANK=IQ\n";
       foreach my $system (sort criterion @{$S}) {
          my @auxSYS = ($system);
          my $IQ = QARLA::IQ($hIQ, $M, $T, \@auxSYS, $R, $verbose);
          printf("%-6s %8.4f\n", $system, $IQ);
       }       
    }
    else { # G == 'seg'
       if ($doOQ) {
          foreach my $m (sort @{$M}) {
             print "RANK=$m\n";
             foreach my $system (sort criterion @{$S}) {
                my $nSEGMENTS = scalar(@{$hOQ->{$m}});
                my $segment = 1;
                while ($segment < $nSEGMENTS) {
                   printf("%-14s %8.4f\n", $system.":".$segment, $hOQ->{$m}->[$segment]->{$system}->{$ref});
                   $segment++;
   	        }
	     }
	  }
       }
       print "RANK=QUEEN\n";
       foreach my $system (sort criterion @{$S}) {
          my $nSEGMENTS = scalar(@{$hIQ->{$M->[0]}});
          my $segment = 1;
          while ($segment < $nSEGMENTS) {
             my @auxSYS = ($system);
             my @auxSEG = ($segment);
             my $QUEEN = QARLA::QUEEN($hIQ, $M, \@auxSEG, \@auxSYS, $R, $verbose);
             printf("%-14s %8.4f\n", $system.":".$segment, $QUEEN);
             $segment++;
          }
       }
       print "RANK=IQ\n";
       foreach my $system (sort criterion @{$S}) {
          my $nSEGMENTS = scalar(@{$hIQ->{$M->[0]}});
          my $segment = 1;
          while ($segment < $nSEGMENTS) {
             my @auxSYS = ($system);
             my @auxSEG = ($segment);
             my $IQ = QARLA::IQ($hIQ, $M, \@auxSEG, \@auxSYS, $R, $verbose);
             printf("%-14s %8.4f\n", $system.":".$segment, $IQ);
             $segment++;
          }
       }       
    }
}

1;
